@@ -25,7 +25,7 @@ const KeyringDialog = new Lang.Class({
         this.prompt = new Shell.KeyringPrompt();
         this.prompt.connect('show-password', Lang.bind(this, this._onShowPassword));
         this.prompt.connect('show-confirm', Lang.bind(this, this._onShowConfirm));
-        this.prompt.connect('hide-prompt', Lang.bind(this, this._onHidePrompt));
+        this.prompt.connect('prompt-close', Lang.bind(this, this._onHidePrompt));
 
         let mainContentBox = new St.BoxLayout({ style_class: 'prompt-dialog-main-layout',
                                                 vertical: false });
@@ -63,11 +63,17 @@ const KeyringDialog = new Lang.Class({
 
         this._cancelButton = this.addButton({ label: '',
                                               action: Lang.bind(this, this._onCancelButton),
-                                              key: Clutter.Escape });
+                                              key: Clutter.Escape },
+                                            { expand: true, x_fill: false, x_align: St.Align.START });
+        this.placeSpinner({ expand: false,
+                            x_fill: false,
+                            y_fill: false,
+                            x_align: St.Align.END,
+                            y_align: St.Align.MIDDLE });
         this._continueButton = this.addButton({ label: '',
                                                 action: Lang.bind(this, this._onContinueButton),
                                                 default: true },
-                                              { expand: true, x_fill: false, x_align: St.Align.END });
+                                              { expand: false, x_fill: false, x_align: St.Align.END });
 
         this.prompt.bind_property('cancel-label', this._cancelButton, 'label', GObject.BindingFlags.SYNC_CREATE);
         this.prompt.bind_property('continue-label', this._continueButton, 'label', GObject.BindingFlags.SYNC_CREATE);
@@ -143,11 +149,19 @@ const KeyringDialog = new Lang.Class({
     },
 
     _updateSensitivity: function(sensitive) {
-        this._passwordEntry.reactive = sensitive;
-        this._passwordEntry.clutter_text.editable = sensitive;
+        if (this._passwordEntry) {
+            this._passwordEntry.reactive = sensitive;
+            this._passwordEntry.clutter_text.editable = sensitive;
+        }
+
+        if (this._confirmEntry) {
+            this._confirmEntry.reactive = sensitive;
+            this._confirmEntry.clutter_text.editable = sensitive;
+        }
 
         this._continueButton.can_focus = sensitive;
         this._continueButton.reactive = sensitive;
+        this.setWorking(!sensitive);
     },
 
     _ensureOpen: function() {
