@@ -49,11 +49,12 @@ const DateMenuButton = new Lang.Class({
             menuAlignment = 1.0 - menuAlignment;
         this.parent(menuAlignment);
 
-        this._clockDisplay = new St.Label();
+        this._clockDisplay = new St.Label({ y_align: Clutter.ActorAlign.CENTER });
         this.actor.add_actor(this._clockDisplay);
+        this.actor.add_style_class_name ('clock-display');
 
-        hbox = new St.BoxLayout({name: 'calendarArea' });
-        this.menu.addActor(hbox);
+        hbox = new St.BoxLayout({ name: 'calendarArea' });
+        this.menu.box.add_child(hbox);
 
         // Fill up the first column
 
