@@ -176,13 +176,16 @@ const IconGrid = new Lang.Class({
     _init: function(params) {
         params = Params.parse(params, { rowLimit: null,
                                         columnLimit: null,
+                                        fillParent: false,
                                         xAlign: St.Align.MIDDLE });
         this._rowLimit = params.rowLimit;
         this._colLimit = params.columnLimit;
         this._xAlign = params.xAlign;
+        this._fillParent = params.fillParent;
 
         this.actor = new St.BoxLayout({ style_class: 'icon-grid',
                                         vertical: true });
+
         // Pulled from CSS, but hardcode some defaults here
         this._spacing = 0;
         this._hItemSize = this._vItemSize = ICON_SIZE;
@@ -196,10 +199,15 @@ const IconGrid = new Lang.Class({
     },
 
     _getPreferredWidth: function (grid, forHeight, alloc) {
-        let children = this._grid.get_children();
+        if (this._fillParent)
+            // Ignore all size requests of children and request a size of 0;
+            // later we'll allocate as many children as fit the parent
+            return;
+
+        let nChildren = this._grid.get_n_children();
         let nColumns = this._colLimit ? Math.min(this._colLimit,
-                                                 children.length)
-                                      : children.length;
+                                                 nChildren)
+                                      : nChildren;
         let totalSpacing = Math.max(0, nColumns - 1) * this._spacing;
         // Kind of a lie, but not really an issue right now.  If
         // we wanted to support some sort of hidden/overflow that would
@@ -217,12 +225,20 @@ const IconGrid = new Lang.Class({
     },
 
     _getPreferredHeight: function (grid, forWidth, alloc) {
+        if (this._fillParent)
+            // Ignore all size requests of children and request a size of 0;
+            // later we'll allocate as many children as fit the parent
+            return;
+
         let children = this._getVisibleChildren();
-        let nColumns;
-        if (forWidth < 0)
+        let nColumns, spacing;
+        if (forWidth < 0) {
             nColumns = children.length;
-        else
-            nColumns = this._computeLayout(forWidth)[0];
+            spacing = this._spacing;
+        } else {
+            [nColumns, , spacing] = this._computeLayout(forWidth);
+        }
+
         let nRows;
         if (nColumns > 0)
             nRows = Math.ceil(children.length / nColumns);
@@ -230,13 +246,20 @@ const IconGrid = new Lang.Class({
             nRows = 0;
         if (this._rowLimit)
             nRows = Math.min(nRows, this._rowLimit);
-        let totalSpacing = Math.max(0, nRows - 1) * this._spacing;
+        let totalSpacing = Math.max(0, nRows - 1) * spacing;
         let height = nRows * this._vItemSize + totalSpacing;
         alloc.min_size = height;
         alloc.natural_size = height;
     },
 
     _allocate: function (grid, box, flags) {
+        if (this._fillParent) {
+            // Reset the passed in box to fill the parent
+            let parentBox = this.actor.get_parent().allocation;
+            let gridBox = this.actor.get_theme_node().get_content_box(parentBox);
+            box = this._grid.get_theme_node().get_content_box(gridBox);
+        }
+
         let children = this._getVisibleChildren();
         let availWidth = box.x2 - box.x1;
         let availHeight = box.y2 - box.y1;
@@ -280,7 +303,8 @@ const IconGrid = new Lang.Class({
             childBox.x2 = childBox.x1 + width;
             childBox.y2 = childBox.y1 + height;
 
-            if (this._rowLimit && rowIndex >= this._rowLimit) {
+            if (this._rowLimit && rowIndex >= this._rowLimit ||
+                this._fillParent && childBox.y2 > availHeight) {
                 this._grid.set_skip_paint(children[i], true);
             } else {
                 children[i].allocate(childBox, flags);
