@@ -45,24 +45,19 @@ const PopupBaseMenuItem = new Lang.Class({
                                          style_class: null,
                                          can_focus: true
                                        });
-        this.actor = new Shell.GenericContainer({ style_class: 'popup-menu-item',
-                                                  reactive: params.reactive,
-                                                  track_hover: params.reactive,
-                                                  can_focus: params.can_focus,
-                                                  accessible_role: Atk.Role.MENU_ITEM});
-        this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
-        this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
-        this.actor.connect('allocate', Lang.bind(this, this._allocate));
-        this.actor.connect('style-changed', Lang.bind(this, this._onStyleChanged));
+
+        this.actor = new St.BoxLayout({ style_class: 'popup-menu-item',
+                                        reactive: params.reactive,
+                                        track_hover: params.reactive,
+                                        can_focus: params.can_focus,
+                                        accessible_role: Atk.Role.MENU_ITEM });
         this.actor._delegate = this;
 
-        this._parent = null;
-        this._children = [];
         this._ornament = Ornament.NONE;
         this._ornamentLabel = new St.Label({ style_class: 'popup-menu-ornament' });
-        this.actor.add_actor(this._ornamentLabel);
-        this._columnWidths = null;
-        this._spacing = 0;
+        this.actor.add(this._ornamentLabel);
+
+        this._parent = null;
         this.active = false;
         this._activatable = params.reactive && params.activate;
         this._sensitive = true;
@@ -95,10 +90,6 @@ const PopupBaseMenuItem = new Lang.Class({
         this._parent = parent;
     },
 
-    _onStyleChanged: function (actor) {
-        this._spacing = Math.round(actor.get_theme_node().get_length('spacing'));
-    },
-
     _onButtonReleaseEvent: function (actor, event) {
         this.activate(event);
         return true;
@@ -170,31 +161,6 @@ const PopupBaseMenuItem = new Lang.Class({
         this.emit('destroy');
     },
 
-    // adds an actor to the menu item; @params can contain
-    // %expand (defaults to #false), and %align (defaults to
-    // #St.Align.START)
-    addActor: function(child, params) {
-        params = Params.parse(params, { expand: false, align: St.Align.START });
-        params.actor = child;
-        this._children.push(params);
-        this.actor.connect('destroy', Lang.bind(this, function () { this._removeChild(child); }));
-        this.actor.add_actor(child);
-    },
-
-    _removeChild: function(child) {
-        for (let i = 0; i < this._children.length; i++) {
-            if (this._children[i].actor == child) {
-                this._children.splice(i, 1);
-                return;
-            }
-        }
-    },
-
-    removeActor: function(child) {
-        this.actor.remove_actor(child);
-        this._removeChild(child);
-    },
-
     setOrnament: function(ornament) {
         if (ornament == this._ornament)
             return;
@@ -211,132 +177,6 @@ const PopupBaseMenuItem = new Lang.Class({
             this._ornamentLabel.text = '';
             this.actor.remove_accessible_state(Atk.StateType.CHECKED);
         }
-    },
-
-    // This returns column widths in logical order (i.e. from the dot
-    // to the image), not in visual order (left to right)
-    getColumnWidths: function() {
-        let widths = [];
-        for (let i = 0, col = 0; i < this._children.length; i++) {
-            let child = this._children[i];
-            let [min, natural] = child.actor.get_preferred_width(-1);
-            widths[col++] = natural;
-        }
-        return widths;
-    },
-
-    setColumnWidths: function(widths) {
-        this._columnWidths = widths;
-    },
-
-    _getPreferredWidth: function(actor, forHeight, alloc) {
-        let width = 0;
-        if (this._columnWidths) {
-            for (let i = 0; i < this._columnWidths.length; i++) {
-                if (i > 0)
-                    width += this._spacing;
-                width += this._columnWidths[i];
-            }
-        } else {
-            for (let i = 0; i < this._children.length; i++) {
-                let child = this._children[i];
-                if (i > 0)
-                    width += this._spacing;
-                let [min, natural] = child.actor.get_preferred_width(-1);
-                width += natural;
-            }
-        }
-        alloc.min_size = alloc.natural_size = width;
-    },
-
-    _getPreferredHeight: function(actor, forWidth, alloc) {
-        let height = 0, x = 0, minWidth, childWidth;
-        for (let i = 0; i < this._children.length; i++) {
-            let child = this._children[i];
-            if (this._columnWidths) {
-                if (child.expand) {
-                    childWidth = 0;
-                    for (let j = i; j < this._columnWidths.length; j++)
-                        childWidth += this._columnWidths[j];
-                } else
-                    childWidth = this._columnWidths[i];
-            } else {
-                if (child.expand)
-                    childWidth = forWidth - x;
-                else
-                    [minWidth, childWidth] = child.actor.get_preferred_width(-1);
-            }
-            x += childWidth;
-
-            let [min, natural] = child.actor.get_preferred_height(childWidth);
-            if (natural > height)
-                height = natural;
-        }
-        alloc.min_size = alloc.natural_size = height;
-    },
-
-    _allocate: function(actor, box, flags) {
-        let width = box.x2 - box.x1;
-        let height = box.y2 - box.y1;
-        let direction = this.actor.get_text_direction();
-
-        // The ornament is placed outside box
-        // one quarter of padding from the border of the container
-        // (so 3/4 from the inner border)
-        // (padding is box.x1)
-        let ornamentBox = new Clutter.ActorBox();
-        let ornamentWidth = box.x1;
-
-        ornamentBox.x1 = 0;
-        ornamentBox.x2 = ornamentWidth;
-        ornamentBox.y1 = box.y1;
-        ornamentBox.y2 = box.y2;
-
-        if (direction == Clutter.TextDirection.RTL) {
-            ornamentBox.x1 += box.x2;
-            ornamentBox.x2 += box.x2;
-        }
-
-        this._ornamentLabel.allocate(ornamentBox, flags);
-
-        let x = box.x1;
-        for (let i = 0, col = 0; i < this._children.length; i++) {
-            let child = this._children[i];
-            let childBox = new Clutter.ActorBox();
-
-            let [minWidth, naturalWidth] = child.actor.get_preferred_width(-1);
-
-            let availWidth;
-            if (child.expand) {
-                availWidth = box.x2 - x;
-            } else {
-                if (this._columnWidths)
-                    availWidth = this._columnWidths[col++];
-                else
-                    availWidth = naturalWidth;
-            }
-
-            if (direction == Clutter.TextDirection.RTL)
-                childBox.x1 = box.x1 + (box.x2 - x - availWidth);
-            else
-                childBox.x1 = x;
-            childBox.x2 = childBox.x1 + availWidth;
-
-            let [minHeight, naturalHeight] = child.actor.get_preferred_height(childBox.x2 - childBox.x1);
-            childBox.y1 = Math.round(box.y1 + (height - naturalHeight) / 2);
-            childBox.y2 = childBox.y1 + naturalHeight;
-
-            let [xAlign, yAlign] = St.get_align_factors(child.align,
-                                                        St.Align.MIDDLE);
-
-            // It's called "expand", but it's really more like "fill"
-            let xFill = child.expand;
-            child.actor.allocate_align_fill(childBox,
-                                            xAlign, yAlign,
-                                            xFill, true,
-                                            flags);
-            x += availWidth + this._spacing;
-        }
     }
 });
 Signals.addSignalMethods(PopupBaseMenuItem.prototype);
@@ -349,7 +189,7 @@ const PopupMenuItem = new Lang.Class({
         this.parent(params);
 
         this.label = new St.Label({ text: text });
-        this.addActor(this.label);
+        this.actor.add_child(this.label);
         this.actor.label_actor = this.label
     }
 });
@@ -362,114 +202,12 @@ const PopupSeparatorMenuItem = new Lang.Class({
         this.parent({ reactive: false,
                       can_focus: false});
 
-        this._box = new St.BoxLayout();
-        this.addActor(this._box, { expand: true });
-
         this.label = new St.Label({ text: text || '' });
-        this._box.add(this.label);
+        this.actor.add(this.label);
         this.actor.label_actor = this.label;
 
         this._separator = new Separator.HorizontalSeparator({ style_class: 'popup-separator-menu-item' });
-        this._box.add(this._separator.actor, { expand: true });
-    }
-});
-
-const PopupAlternatingMenuItemState = {
-    DEFAULT: 0,
-    ALTERNATIVE: 1
-}
-
-const PopupAlternatingMenuItem = new Lang.Class({
-    Name: 'PopupAlternatingMenuItem',
-    Extends: PopupBaseMenuItem,
-
-    _init: function(text, alternateText, params) {
-        this.parent(params);
-        this.actor.add_style_class_name('popup-alternating-menu-item');
-
-        this._text = text;
-        this._alternateText = alternateText;
-        this.label = new St.Label({ text: text });
-        this.state = PopupAlternatingMenuItemState.DEFAULT;
-        this.addActor(this.label);
-        this.actor.label_actor = this.label;
-
-        this.actor.connect('notify::mapped', Lang.bind(this, this._onMapped));
-    },
-
-    _onMapped: function() {
-        if (this.actor.mapped) {
-            this._capturedEventId = global.stage.connect('captured-event',
-                                                         Lang.bind(this, this._onCapturedEvent));
-            this._updateStateFromModifiers();
-        } else {
-            if (this._capturedEventId != 0) {
-                global.stage.disconnect(this._capturedEventId);
-                this._capturedEventId = 0;
-            }
-        }
-    },
-
-    _setState: function(state) {
-        if (this.state != state) {
-            if (state == PopupAlternatingMenuItemState.ALTERNATIVE && !this._canAlternate())
-                return;
-
-            this.state = state;
-            this._updateLabel();
-        }
-    },
-
-    _updateStateFromModifiers: function() {
-        let [x, y, mods] = global.get_pointer();
-        let state;
-
-        if ((mods & Clutter.ModifierType.MOD1_MASK) == 0) {
-            state = PopupAlternatingMenuItemState.DEFAULT;
-        } else {
-            state = PopupAlternatingMenuItemState.ALTERNATIVE;
-        }
-
-        this._setState(state);
-    },
-
-    _onCapturedEvent: function(actor, event) {
-        if (event.type() != Clutter.EventType.KEY_PRESS &&
-            event.type() != Clutter.EventType.KEY_RELEASE)
-            return false;
-
-        let key = event.get_key_symbol();
-
-        if (key == Clutter.KEY_Alt_L || key == Clutter.KEY_Alt_R)
-            this._updateStateFromModifiers();
-
-        return false;
-    },
-
-    _updateLabel: function() {
-        if (this.state == PopupAlternatingMenuItemState.ALTERNATIVE) {
-            this.actor.add_style_pseudo_class('alternate');
-            this.label.set_text(this._alternateText);
-        } else {
-            this.actor.remove_style_pseudo_class('alternate');
-            this.label.set_text(this._text);
-        }
-    },
-
-    _canAlternate: function() {
-        if (this.state == PopupAlternatingMenuItemState.DEFAULT && !this._alternateText)
-            return false;
-        return true;
-    },
-
-    updateText: function(text, alternateText) {
-        this._text = text;
-        this._alternateText = alternateText;
-
-        if (!this._canAlternate())
-            this._setState(PopupAlternatingMenuItemState.DEFAULT);
-
-        this._updateLabel();
+        this.actor.add(this._separator.actor, { expand: true });
     }
 });
 
@@ -516,10 +254,10 @@ const PopupSwitchMenuItem = new Lang.Class({
         this.checkAccessibleState();
         this.actor.label_actor = this.label;
 
-        this.addActor(this.label);
+        this.actor.add_child(this.label);
 
         this._statusBin = new St.Bin({ x_align: St.Align.END });
-        this.addActor(this._statusBin, { expand: true, align: St.Align.END });
+        this.actor.add(this._statusBin, { expand: true, x_align: St.Align.END });
 
         this._statusLabel = new St.Label({ text: '',
                                            style_class: 'popup-status-menu-item'
@@ -618,7 +356,6 @@ const PopupMenuBase = new Lang.Class({
         } else {
             this.box = new St.BoxLayout({ vertical: true });
         }
-        this.box.connect_after('queue-relayout', Lang.bind(this, this._menuQueueRelayout));
         this.length = 0;
 
         this.isOpen = false;
@@ -819,9 +556,6 @@ const PopupMenuBase = new Lang.Class({
         }
 
         if (menuItem instanceof PopupMenuSection) {
-            let activateId = menuItem.connect('activate', Lang.bind(this, function() {
-                this.emit('activate');
-            }));
             let activeChangeId = menuItem.connect('active-changed', Lang.bind(this, this._subMenuActiveChanged));
 
             let parentOpenStateChangedId = this.connect('open-state-changed', function(self, open) {
@@ -838,7 +572,6 @@ const PopupMenuBase = new Lang.Class({
             }));
 
             menuItem.connect('destroy', Lang.bind(this, function() {
-                menuItem.disconnect(activateId);
                 menuItem.disconnect(activeChangeId);
                 this.disconnect(subMenuSensitiveChangedId);
                 this.disconnect(parentOpenStateChangedId);
@@ -852,16 +585,12 @@ const PopupMenuBase = new Lang.Class({
                 this.box.insert_child_below(menuItem.menu.actor, before_item);
 
             this._connectItemSignals(menuItem);
-            let subMenuActivateId = menuItem.connect('activate', Lang.bind(this, function() {
-                this.emit('activate');
-            }));
             let subMenuActiveChangeId = menuItem.menu.connect('active-changed', Lang.bind(this, this._subMenuActiveChanged));
             let closingId = this.connect('menu-closed', function() {
                 menuItem.menu.close(BoxPointer.PopupAnimation.NONE);
             });
 
             menuItem.connect('destroy', Lang.bind(this, function() {
-                menuItem.menu.disconnect(subMenuActivateId);
                 menuItem.menu.disconnect(subMenuActiveChangeId);
                 this.disconnect(closingId);
             }));
@@ -872,7 +601,11 @@ const PopupMenuBase = new Lang.Class({
             // separator's adjacent siblings change visibility or position.
             // open-state-changed isn't exactly that, but doing it in more
             // precise ways would require a lot more bookkeeping.
-            this.connect('open-state-changed', Lang.bind(this, function() { this._updateSeparatorVisibility(menuItem); }));
+            let openStateChangeId = this.connect('open-state-changed', Lang.bind(this, function() { this._updateSeparatorVisibility(menuItem); }));
+            let destroyId = menuItem.connect('destroy', Lang.bind(this, function() {
+                this.disconnect(openStateChangeId);
+                menuItem.disconnect(destroyId);
+            }));
         } else if (menuItem instanceof PopupBaseMenuItem)
             this._connectItemSignals(menuItem);
         else
@@ -883,52 +616,6 @@ const PopupMenuBase = new Lang.Class({
         this.length++;
     },
 
-    getColumnWidths: function() {
-        let columnWidths = [];
-        let items = this.box.get_children();
-        for (let i = 0; i < items.length; i++) {
-            if (!items[i].visible)
-                continue;
-
-            if (items[i]._delegate instanceof PopupSubMenu)
-                continue;
-
-            if (items[i]._delegate instanceof PopupBaseMenuItem || items[i]._delegate instanceof PopupMenuBase) {
-                let itemColumnWidths = items[i]._delegate.getColumnWidths();
-                for (let j = 0; j < itemColumnWidths.length; j++) {
-                    if (j >= columnWidths.length || itemColumnWidths[j] > columnWidths[j])
-                        columnWidths[j] = itemColumnWidths[j];
-                }
-            }
-        }
-        return columnWidths;
-    },
-
-    setColumnWidths: function(widths) {
-        let items = this.box.get_children();
-        for (let i = 0; i < items.length; i++) {
-            if (items[i]._delegate instanceof PopupSubMenu)
-                continue;
-
-            if (items[i]._delegate instanceof PopupBaseMenuItem || items[i]._delegate instanceof PopupMenuBase)
-                items[i]._delegate.setColumnWidths(widths);
-        }
-    },
-
-    // Because of the above column-width funniness, we need to do a
-    // queue-relayout on every item whenever the menu itself changes
-    // size, to force clutter to drop its cached size requests. (The
-    // menuitems will in turn call queue_relayout on their parent, the
-    // menu, but that call will be a no-op since the menu already
-    // has a relayout queued, so we won't get stuck in a loop.
-    _menuQueueRelayout: function() {
-        this.box.get_children().map(function (actor) { actor.queue_relayout(); });
-    },
-
-    addActor: function(actor) {
-        this.box.add(actor);
-    },
-
     _getMenuItems: function() {
         return this.box.get_children().map(function (actor) {
             return actor._delegate;
@@ -995,12 +682,7 @@ const PopupMenu = new Lang.Class({
         this.actor._delegate = this;
         this.actor.style_class = 'popup-menu-boxpointer';
 
-        this._boxWrapper = new Shell.GenericContainer();
-        this._boxWrapper.connect('get-preferred-width', Lang.bind(this, this._boxGetPreferredWidth));
-        this._boxWrapper.connect('get-preferred-height', Lang.bind(this, this._boxGetPreferredHeight));
-        this._boxWrapper.connect('allocate', Lang.bind(this, this._boxAllocate));
-        this._boxPointer.bin.set_child(this._boxWrapper);
-        this._boxWrapper.add_actor(this.box);
+        this._boxPointer.bin.set_child(this.box);
         this.actor.add_style_class_name('popup-menu');
 
         global.focus_manager.add_group(this.actor);
@@ -1016,22 +698,6 @@ const PopupMenu = new Lang.Class({
         this._openedSubMenu = submenu;
     },
 
-    _boxGetPreferredWidth: function (actor, forHeight, alloc) {
-        let columnWidths = this.getColumnWidths();
-        this.setColumnWidths(columnWidths);
-
-        // Now they will request the right sizes
-        [alloc.min_size, alloc.natural_size] = this.box.get_preferred_width(forHeight);
-    },
-
-    _boxGetPreferredHeight: function (actor, forWidth, alloc) {
-        [alloc.min_size, alloc.natural_size] = this.box.get_preferred_height(forWidth);
-    },
-
-    _boxAllocate: function (actor, box, flags) {
-        this.box.allocate(box, flags);
-    },
-
     setArrowOrigin: function(origin) {
         this._boxPointer.setArrowOrigin(origin);
     },
@@ -1105,7 +771,6 @@ const PopupSubMenu = new Lang.Class({
         this.parent(sourceActor);
 
         this._arrow = sourceArrow;
-        this._arrow.rotation_center_z_gravity = Clutter.Gravity.CENTER;
 
         // Since a function of a submenu might be to provide a "More.." expander
         // with long content, we make it scrollable - the scrollbar will only take
@@ -1273,18 +938,30 @@ const PopupSubMenuMenuItem = new Lang.Class({
 
         if (wantIcon) {
             this.icon = new St.Icon({ style_class: 'popup-menu-icon' });
-            this.addActor(this.icon, { align: St.Align.MIDDLE });
+            this.actor.add_child(this.icon);
         }
 
         this.label = new St.Label({ text: text });
-        this.addActor(this.label);
+        this.actor.add_child(this.label);
         this.actor.label_actor = this.label;
 
-        this.status = new St.Label({ style_class: 'popup-status-menu-item' });
-        this.addActor(this.status, { align: St.Align.END });
+        let expander = new St.Bin({ style_class: 'popup-menu-item-expander' });
+        this.actor.add(expander, { expand: true });
 
-        this._triangle = new St.Label({ text: '\u25B8' });
-        this.addActor(this._triangle, { align: St.Align.END });
+        this.status = new St.Label({ style_class: 'popup-status-menu-item' });
+        this.actor.add_child(this.status);
+
+        this._triangle = new St.Label({ text: '\u25B8',
+                                        style_class: 'popup-submenu-menu-item-triangle' });
+        this._triangle.pivot_point = new Clutter.Point({ x: 0.5, y: 0.6 });
+
+        this._triangleBin = new St.Widget({ y_expand: true,
+                                            y_align: Clutter.ActorAlign.CENTER });
+        this._triangleBin.add_child(this._triangle);
+        if (this._triangleBin.get_text_direction() == Clutter.TextDirection.RTL)
+            this._triangleBin.set_scale(-1.0, 1.0);
+
+        this.actor.add_child(this._triangleBin);
 
         this.menu = new PopupSubMenu(this.actor, this._triangle);
         this.menu.connect('open-state-changed', Lang.bind(this, this._subMenuOpenStateChanged));
