@@ -1,10 +1,12 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
+const AccountsService = imports.gi.AccountsService;
 const Cairo = imports.cairo;
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const GnomeDesktop = imports.gi.GnomeDesktop;
+const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -23,6 +25,7 @@ const Main = imports.ui.main;
 const Overview = imports.ui.overview;
 const MessageTray = imports.ui.messageTray;
 const ShellDBus = imports.ui.shellDBus;
+const SmartcardManager = imports.misc.smartcardManager;
 const Tweener = imports.ui.tweener;
 const Util = imports.misc.util;
 
@@ -104,13 +107,14 @@ const NotificationsBox = new Lang.Class({
         this._musicBin = new St.Bin({ style_class: 'screen-shield-notifications-box',
                                       visible: false });
 
-        let scrollView = new St.ScrollView({ x_fill: false, x_align: St.Align.START });
+        this._scrollView = new St.ScrollView({ x_fill: false, x_align: St.Align.START,
+                                               hscrollbar_policy: Gtk.PolicyType.NEVER });
         this._notificationBox = new St.BoxLayout({ vertical: true,
                                                    style_class: 'screen-shield-notifications-box' });
-        scrollView.add_actor(this._notificationBox);
+        this._scrollView.add_actor(this._notificationBox);
 
         this.actor.add(this._musicBin);
-        this.actor.add(scrollView, { x_fill: true, x_align: St.Align.START });
+        this.actor.add(this._scrollView, { x_fill: true, x_align: St.Align.START });
 
         this._sources = new Hash.Map();
         Main.messageTray.getSources().forEach(Lang.bind(this, function(source) {
@@ -236,7 +240,7 @@ const NotificationsBox = new Lang.Class({
             (source.unseenCount > (musicNotification ? 1 : 0));
     },
 
-    _sourceAdded: function(tray, source, dontUpdateVisibility) {
+    _sourceAdded: function(tray, source, initial) {
         // Ignore transient sources
         if (source.isTransient)
             return;
@@ -283,8 +287,29 @@ const NotificationsBox = new Lang.Class({
 
         this._sources.set(source, obj);
 
-        if (!dontUpdateVisibility)
+        if (!initial) {
+            // block scrollbars while animating, if they're not needed now
+            let boxHeight = this._notificationBox.height;
+            if (this._scrollView.height >= boxHeight)
+                this._scrollView.vscrollbar_policy = Gtk.PolicyType.NEVER;
+
+            let widget = obj.sourceBox;
+            let [, natHeight] = widget.get_preferred_height(-1);
+            widget.height = 0;
+            Tweener.addTween(widget,
+                             { height: natHeight,
+                               transition: 'easeOutQuad',
+                               time: 0.25,
+                               onComplete: function() {
+                                   this._scrollView.vscrollbar_policy = Gtk.PolicyType.AUTOMATIC;
+                                   widget.set_height(-1);
+                               },
+                               onCompleteScope: this
+                             });
+
             this._updateVisibility();
+            Shell.util_wake_up_screen();
+        }
     },
 
     _titleChanged: function(source, obj) {
@@ -306,7 +331,10 @@ const NotificationsBox = new Lang.Class({
 
         obj.sourceBox.visible = obj.visible &&
             (source.unseenCount > (obj.musicNotification ? 1 : 0));
+
         this._updateVisibility();
+        if (obj.sourceBox.visible)
+            Shell.util_wake_up_screen();
     },
 
     _visibleChanged: function(source, obj) {
@@ -320,6 +348,8 @@ const NotificationsBox = new Lang.Class({
             source.unseenCount > (obj.musicNotification ? 1 : 0);
 
         this._updateVisibility();
+        if (obj.sourceBox.visible)
+            Shell.util_wake_up_screen();
     },
 
     _detailedChanged: function(source, obj) {
@@ -489,6 +519,7 @@ const ScreenShield = new Lang.Class({
 
         this._lockDialogGroup = new St.Widget({ x_expand: true,
                                                 y_expand: true,
+                                                reactive: true,
                                                 opacity: 0,
                                                 pivot_point: new Clutter.Point({ x: 0.5, y: 0.5 }),
                                                 name: 'lockDialogGroup' });
@@ -516,6 +547,13 @@ const ScreenShield = new Lang.Class({
 
         this._screenSaverDBus = new ShellDBus.ScreenSaverDBus(this);
 
+        this._smartcardManager = SmartcardManager.getSmartcardManager();
+        this._smartcardManager.connect('smartcard-inserted',
+                                       Lang.bind(this, function(token) {
+                                           if (this._isLocked && token.UsedToLogin)
+                                               this._liftShield(true, 0);
+                                       }));
+
         this._inhibitor = null;
         this._aboutToSuspend = false;
         this._loginManager = LoginManager.getLoginManager();
@@ -542,11 +580,17 @@ const ScreenShield = new Lang.Class({
         this._becameActiveId = 0;
         this._lockTimeoutId = 0;
 
-        this._lightbox = new Lightbox.Lightbox(Main.uiGroup,
-                                               { inhibitEvents: true,
-                                                 fadeInTime: STANDARD_FADE_TIME,
-                                                 fadeFactor: 1 });
-        this._lightbox.connect('shown', Lang.bind(this, this._onLightboxShown));
+        // The "long" lightbox is used for the longer (20 seconds) fade from session
+        // to idle status, the "short" is used for quickly fading to black when locking
+        // manually
+        this._longLightbox = new Lightbox.Lightbox(Main.uiGroup,
+                                                   { inhibitEvents: true,
+                                                     fadeFactor: 1 });
+        this._longLightbox.connect('shown', Lang.bind(this, this._onLongLightboxShown));
+        this._shortLightbox = new Lightbox.Lightbox(Main.uiGroup,
+                                                    { inhibitEvents: true,
+                                                      fadeFactor: 1 });
+        this._shortLightbox.connect('shown', Lang.bind(this, this._onShortLightboxShown));
 
         this.idleMonitor = new GnomeDesktop.IdleMonitor();
     },
@@ -561,7 +605,8 @@ const ScreenShield = new Lang.Class({
 
         let bgManager = new Background.BackgroundManager({ container: widget,
                                                            monitorIndex: monitorIndex,
-                                                           controlPosition: false });
+                                                           controlPosition: false,
+                                                           settingsSchema: SCREENSAVER_SCHEMA });
 
         this._bgManagers.push(bgManager);
 
@@ -581,13 +626,28 @@ const ScreenShield = new Lang.Class({
 
     _liftShield: function(onPrimary, velocity) {
         if (this._isLocked) {
-            this._ensureUnlockDialog(onPrimary, true /* allowCancel */);
-            this._hideLockScreen(true /* animate */, velocity);
+            if (this._ensureUnlockDialog(onPrimary, true /* allowCancel */))
+                this._hideLockScreen(true /* animate */, velocity);
         } else {
             this.deactivate(true /* animate */);
         }
     },
 
+    _maybeCancelDialog: function() {
+        if (!this._dialog)
+            return;
+
+        this._dialog.cancel();
+        if (this._isGreeter) {
+            // LoginDialog.cancel() will grab the key focus
+            // on its own, so ensure it stays on lock screen
+            // instead
+            this._lockScreenGroup.grab_key_focus();
+        } else {
+            this._dialog = null;
+        }
+    },
+
     _becomeModal: function() {
         if (this._isModal)
             return true;
@@ -619,9 +679,9 @@ const ScreenShield = new Lang.Class({
         if (!isEnter && !(GLib.unichar_isprint(unichar) || symbol == Clutter.KEY_Escape))
             return false;
 
-        this._ensureUnlockDialog(true, true);
-
-        if (GLib.unichar_isgraph(unichar))
+        if (this._isLocked &&
+            this._ensureUnlockDialog(true, true) &&
+            GLib.unichar_isgraph(unichar))
             this._dialog.addCharacter(unichar);
 
         this._liftShield(true, 0);
@@ -743,13 +803,7 @@ const ScreenShield = new Lang.Class({
                                onCompleteScope: this,
                              });
 
-            // If we have a unlock dialog, cancel it
-            if (this._dialog) {
-                this._dialog.cancel();
-                if (!this._isGreeter) {
-                    this._dialog = null;
-                }
-            }
+            this._maybeCancelDialog();
         }
     },
 
@@ -757,14 +811,9 @@ const ScreenShield = new Lang.Class({
         if (status != GnomeSession.PresenceStatus.IDLE)
             return;
 
-        if (this._dialog) {
-            this._dialog.cancel();
-            if (!this._isGreeter) {
-                this._dialog = null;
-            }
-        }
+        this._maybeCancelDialog();
 
-        if (this._lightbox.actor.visible ||
+        if (this._longLightbox.actor.visible ||
             this._isActive) {
             // We're either shown and active, or in the process of
             // showing.
@@ -782,7 +831,7 @@ const ScreenShield = new Lang.Class({
             // screenshield. The user is probably very upset at this
             // point, but any application using global grabs is broken
             // Just tell him to stop using this app
-            // 
+            //
             // XXX: another option is to kick the user into the gdm login
             // screen, where we're not affected by grabs
             Main.notifyError(_("Unable to lock"),
@@ -790,13 +839,9 @@ const ScreenShield = new Lang.Class({
             return;
         }
 
-        this._lightbox.show();
-
         if (this._activationTime == 0)
             this._activationTime = GLib.get_monotonic_time();
 
-        this._becameActiveId = this.idleMonitor.add_user_active_watch(Lang.bind(this, this._onUserBecameActive));
-
         let shouldLock = this._settings.get_boolean(LOCK_ENABLED_KEY) && !this._isLocked;
 
         if (shouldLock) {
@@ -804,48 +849,59 @@ const ScreenShield = new Lang.Class({
             this._lockTimeoutId = Mainloop.timeout_add(lockTimeout * 1000,
                                                        Lang.bind(this, function() {
                                                            this._lockTimeoutId = 0;
-                                                           this.lock(true);
+                                                           this.lock(false);
                                                            return false;
                                                        }));
         }
+
+        this._activateFade(this._longLightbox, STANDARD_FADE_TIME);
+    },
+
+    _activateFade: function(lightbox, time) {
+        lightbox.show(time);
+
+        if (this._becameActiveId == 0)
+            this._becameActiveId = this.idleMonitor.add_user_active_watch(Lang.bind(this, this._onUserBecameActive));
     },
 
     _onUserBecameActive: function() {
         // This function gets called here when the user becomes active
-        // after gnome-session changed the status to IDLE
-        // There are four possibilities here:
-        // - we're called when already locked; isActive and isLocked are true,
+        // after we activated a lightbox
+        // There are two possibilities here:
+        // - we're called when already locked/active; isLocked or isActive is true,
         //   we just go back to the lock screen curtain
-        // - we're called before the lightbox is fully shown; at this point
-        //   isActive is false, so we just hide the ligthbox, reset the activationTime
-        //   and go back to the unlocked desktop
-        // - we're called after showing the lightbox, but before the lock
-        //   delay; this is mostly like the case above, but isActive is true now
-        //   so we need to notify gnome-settings-daemon to go back to the normal
-        //   policies for blanking
-        //   (they're handled by the same code, and we emit one extra ActiveChanged
-        //   signal in the case above)
-        // - we're called after showing the lightbox and after lock-delay; the
-        //   session is effectivelly locked now, it's time to build and show
-        //   the lock screen
+        //   (isActive == isLocked == true: normal case
+        //    isActive == false, isLocked == true: during the fade for manual locking
+        //    isActive == true, isLocked == false: after session idle, before lock-delay)
+        // - we're called because the session is IDLE but before the lightbox
+        //   is fully shown; at this point isActive is false, so we just hide
+        //   the lightbox, reset the activationTime and go back to the unlocked
+        //   desktop
+        //   using deactivate() is a little of overkill, but it ensures we
+        //   don't forget of some bit like modal, DBus properties or idle watches
+        //
+        // Note: if the (long) lightbox is shown then we're necessarily
+        // active, because we call activate() without animation.
 
         this.idleMonitor.remove_watch(this._becameActiveId);
         this._becameActiveId = 0;
 
-        let lightboxWasShown = this._lightbox.shown;
-        this._lightbox.hide();
-
-        // Shortcircuit in case the mouse was moved before the fade completed
-        if (!lightboxWasShown) {
+        if (this._isActive || this._isLocked) {
+            this._longLightbox.hide();
+            this._shortLightbox.hide();
+        } else {
             this.deactivate(false);
-            return;
         }
     },
 
-    _onLightboxShown: function() {
+    _onLongLightboxShown: function() {
         this.activate(false);
     },
 
+    _onShortLightboxShown: function() {
+        this._completeLockScreenShown();
+    },
+
     showDialog: function() {
         // Ensure that the stage window is mapped, before taking a grab
         // otherwise X errors out
@@ -862,8 +918,8 @@ const ScreenShield = new Lang.Class({
         this.actor.show();
         this._isGreeter = Main.sessionMode.isGreeter;
         this._isLocked = true;
-        this._ensureUnlockDialog(true, true);
-        this._hideLockScreen(false, 0);
+        if (this._ensureUnlockDialog(true, true))
+            this._hideLockScreen(false, 0);
     },
 
     _hideLockScreenComplete: function() {
@@ -880,6 +936,8 @@ const ScreenShield = new Lang.Class({
 
         this._lockScreenState = MessageTray.State.HIDING;
 
+        Tweener.removeTweens(this._lockScreenGroup);
+
         if (animate) {
             // Tween the lock screen out of screen
             // if velocity is not specified (i.e. we come here from pressing ESC),
@@ -892,7 +950,6 @@ const ScreenShield = new Lang.Class({
             velocity = Math.max(min_velocity, velocity);
             let time = (delta / velocity) / 1000;
 
-            Tweener.removeTweens(this._lockScreenGroup);
             Tweener.addTween(this._lockScreenGroup,
                              { y: -h,
                                time: time,
@@ -912,7 +969,7 @@ const ScreenShield = new Lang.Class({
             if (!constructor) {
                 // This session mode has no locking capabilities
                 this.deactivate(true);
-                return;
+                return false;
             }
 
             this._dialog = new constructor(this._lockDialogGroup);
@@ -924,19 +981,23 @@ const ScreenShield = new Lang.Class({
                 // by the time we reach this...
                 log('Could not open login dialog: failed to acquire grab');
                 this.deactivate(true);
+                return false;
             }
 
             this._dialog.connect('failed', Lang.bind(this, this._onUnlockFailed));
         }
 
         this._dialog.allowCancel = allowCancel;
+        return true;
     },
 
     _onUnlockFailed: function() {
-        this._resetLockScreen(true, false);
+        this._resetLockScreen({ animateLockScreen: true,
+                                animateLockDialog: false,
+                                fadeToBlack: false });
     },
 
-    _resetLockScreen: function(animateLockScreen, animateLockDialog) {
+    _resetLockScreen: function(params) {
         // Don't reset the lock screen unless it is completely hidden
         // This prevents the shield going down if the lock-delay timeout
         // fires while the user is dragging (which has the potential
@@ -951,7 +1012,9 @@ const ScreenShield = new Lang.Class({
         this._lockScreenGroup.show();
         this._lockScreenState = MessageTray.State.SHOWING;
 
-        if (animateLockScreen) {
+        let fadeToBlack = params.fadeToBlack;
+
+        if (params.animateLockScreen) {
             this._lockScreenGroup.y = -global.screen_height;
             Tweener.removeTweens(this._lockScreenGroup);
             Tweener.addTween(this._lockScreenGroup,
@@ -959,16 +1022,18 @@ const ScreenShield = new Lang.Class({
                                time: MANUAL_FADE_TIME,
                                transition: 'easeOutQuad',
                                onComplete: function() {
-                                   this._lockScreenShown();
+                                   this._lockScreenShown({ fadeToBlack: fadeToBlack,
+                                                           animateFade: true });
                                },
                                onCompleteScope: this
                              });
         } else {
             this._lockScreenGroup.fixed_position_set = false;
-            this._lockScreenShown();
+            this._lockScreenShown({ fadeToBlack: fadeToBlack,
+                                    animateFade: false });
         }
 
-        if (animateLockDialog) {
+        if (params.animateLockDialog) {
             this._lockDialogGroup.opacity = 0;
             Tweener.removeTweens(this._lockDialogGroup);
             Tweener.addTween(this._lockDialogGroup,
@@ -1032,7 +1097,7 @@ const ScreenShield = new Lang.Class({
             this._pauseArrowAnimation();
     },
 
-    _lockScreenShown: function() {
+    _lockScreenShown: function(params) {
         if (this._dialog && !this._isGreeter) {
             this._dialog.destroy();
             this._dialog = null;
@@ -1054,6 +1119,21 @@ const ScreenShield = new Lang.Class({
         this._lockScreenGroup.fixed_position_set = false;
         this._lockScreenScrollCounter = 0;
 
+        if (params.fadeToBlack && params.animateFade) {
+            // Take a beat
+
+            Mainloop.timeout_add(1000 * MANUAL_FADE_TIME, Lang.bind(this, function() {
+                this._activateFade(this._shortLightbox, MANUAL_FADE_TIME);
+            }));
+        } else {
+            if (params.fadeToBlack)
+                this._activateFade(this._shortLightbox, 0);
+
+            this._completeLockScreenShown();
+        }
+    },
+
+    _completeLockScreenShown: function() {
         let prevIsActive = this._isActive;
         this._isActive = true;
 
@@ -1121,12 +1201,15 @@ const ScreenShield = new Lang.Class({
     },
 
     deactivate: function(animate) {
-        this._dialog.finish(Lang.bind(this, function() {
-            this._finishDeactivate(animate);
-        }));
+        if (this._dialog)
+            this._dialog.finish(Lang.bind(this, function() {
+                this._continueDeactivate(animate);
+            }));
+        else
+            this._continueDeactivate(animate);
     },
 
-    _finishDeactivate: function(animate) {
+    _continueDeactivate: function(animate) {
         this._hideLockScreen(animate, 0);
 
         if (this._hasLockScreen)
@@ -1137,6 +1220,20 @@ const ScreenShield = new Lang.Class({
         if (Main.sessionMode.currentMode == 'unlock-dialog')
             Main.sessionMode.popMode('unlock-dialog');
 
+        if (this._isGreeter) {
+            // We don't want to "deactivate" any more than
+            // this. In particular, we don't want to drop
+            // the modal, hide ourselves or destroy the dialog
+            // But we do want to set isActive to false, so that
+            // gnome-session will reset the idle counter, and
+            // gnome-settings-daemon will stop blanking the screen
+
+            this._activationTime = 0;
+            this._isActive = false;
+            this.emit('active-changed');
+            return;
+        }
+
         if (this._dialog && !this._isGreeter)
             this._dialog.popModal();
 
@@ -1156,12 +1253,13 @@ const ScreenShield = new Lang.Class({
     },
 
     _completeDeactivate: function() {
-        if (this._dialog && !this._isGreeter) {
+        if (this._dialog) {
             this._dialog.destroy();
             this._dialog = null;
         }
 
-        this._lightbox.hide();
+        this._longLightbox.hide();
+        this._shortLightbox.hide();
         this.actor.hide();
 
         if (this._becameActiveId != 0) {
@@ -1195,7 +1293,9 @@ const ScreenShield = new Lang.Class({
                 Main.sessionMode.pushMode('unlock-dialog');
         }
 
-        this._resetLockScreen(animate, animate);
+        this._resetLockScreen({ animateLockScreen: animate,
+                                animateLockDialog: animate,
+                                fadeToBlack: true });
         global.set_runtime_state(LOCKED_STATE_STR, GLib.Variant.new('b', true));
 
         // We used to set isActive and emit active-changed here,
@@ -1224,7 +1324,14 @@ const ScreenShield = new Lang.Class({
         St.Clipboard.get_default().set_text(St.ClipboardType.CLIPBOARD, '');
         St.Clipboard.get_default().set_text(St.ClipboardType.PRIMARY, '');
 
-        this._isLocked = true;
+        let userManager = AccountsService.UserManager.get_default();
+        let user = userManager.get_user(GLib.get_user_name());
+
+        if (this._isGreeter)
+            this._isLocked = true;
+        else
+            this._isLocked = user.password_mode != AccountsService.UserPasswordMode.NONE;
+
         this.activate(animate);
 
         this.emit('locked-changed');
