@@ -187,7 +187,7 @@ const NMConnectionSection = new Lang.Class({
 
     _getStatus: function() {
         let values = this._connectionItems.values();
-        for (let i = 0; i < values; i++) {
+        for (let i = 0; i < values.length; i++) {
             let item = values[i];
             if (item.isActive())
                 return item.getName();
@@ -524,18 +524,21 @@ const NMWirelessDialogItem = new Lang.Class({
         this._label = new St.Label({ text: title });
 
         this.actor.label_actor = this._label;
-        this._content.add(this._label, { expand: true, x_align: St.Align.START });
+        this._content.add(this._label, { x_align: St.Align.START });
+
+        this._selectedIcon = new St.Icon({ style_class: 'nm-dialog-icon',
+                                           icon_name: 'object-select-symbolic' });
+        this._content.add(this._selectedIcon);
 
         this._icons = new St.BoxLayout({ style_class: 'nm-dialog-icons' });
-        this._content.add(this._icons, { x_fill: false, x_align: St.Align.END });
+        this._content.add(this._icons, { expand: true, x_fill: false, x_align: St.Align.END });
 
         this._secureIcon = new St.Icon({ style_class: 'nm-dialog-icon' });
         if (this._ap._secType != NMAccessPointSecurity.NONE)
             this._secureIcon.icon_name = 'network-wireless-encrypted-symbolic';
         this._icons.add_actor(this._secureIcon);
 
-        this._signalIcon = new St.Icon({ icon_name: this._getIcon(),
-                                         style_class: 'nm-dialog-icon' });
+        this._signalIcon = new St.Icon({ style_class: 'nm-dialog-icon' });
         this._icons.add_actor(this._signalIcon);
     },
 
@@ -544,6 +547,10 @@ const NMWirelessDialogItem = new Lang.Class({
         this._signalIcon.icon_name = this._getIcon();
     },
 
+    setActive: function(isActive) {
+        this._selectedIcon.opacity = isActive ? 255 : 0;
+    },
+
     _getIcon: function() {
         if (this._ap.mode == NM80211Mode.ADHOC)
             return 'network-workgroup-symbolic';
@@ -573,6 +580,7 @@ const NMWirelessDialog = new Lang.Class({
 
         this._apAddedId = device.connect('access-point-added', Lang.bind(this, this._accessPointAdded));
         this._apRemovedId = device.connect('access-point-removed', Lang.bind(this, this._accessPointRemoved));
+        this._activeApChangedId = device.connect('notify::active-access-point', Lang.bind(this, this._activeApChanged));
 
         // accessPointAdded will also create dialog items
         let accessPoints = device.get_access_points() || [ ];
@@ -581,6 +589,7 @@ const NMWirelessDialog = new Lang.Class({
         }));
 
         this._selectedNetwork = null;
+        this._activeApChanged();
         this._updateSensitivity();
     },
 
@@ -589,21 +598,44 @@ const NMWirelessDialog = new Lang.Class({
             GObject.Object.prototype.disconnect.call(this._device, this._apAddedId);
             this._apAddedId = 0;
         }
-
         if (this._apRemovedId) {
             GObject.Object.prototype.disconnect.call(this._device, this._apRemovedId);
             this._apRemovedId = 0;
         }
+        if (this._activeApChangedId) {
+            GObject.Object.prototype.disconnect.call(this._device, this._activeApChangedId);
+            this._activeApChangedId = 0;
+        }
 
         this.parent();
     },
 
+    _activeApChanged: function() {
+        if (this._activeNetwork)
+            this._activeNetwork.item.setActive(false);
+
+        this._activeNetwork = null;
+        if (this._device.active_access_point) {
+            let idx = this._findNetwork(this._device.active_access_point);
+            if (idx >= 0)
+                this._activeNetwork = this._networks[idx];
+        }
+
+        if (this._activeNetwork)
+            this._activeNetwork.item.setActive(true);
+        this._updateSensitivity();
+    },
+
     _updateSensitivity: function() {
-        let connectSensitive = this._selectedNetwork != null;
+        let connectSensitive = this._selectedNetwork && (this._selectedNetwork != this._activeNetwork);
         this._connectButton.reactive = connectSensitive;
         this._connectButton.can_focus = connectSensitive;
     },
 
+    _updateVisibility: function() {
+        this._noNetworksLabel.visible = (this._networks.length == 0);
+    },
+
     _buildLayout: function() {
         let headline = new St.BoxLayout({ style_class: 'nm-dialog-header-hbox' });
 
@@ -624,12 +656,24 @@ const NMWirelessDialog = new Lang.Class({
         this.contentLayout.style_class = 'nm-dialog-content';
         this.contentLayout.add(headline);
 
+        this._stack = new St.Widget({ layout_manager: new Clutter.BinLayout() });
+
         this._itemBox = new St.BoxLayout({ vertical: true });
         this._scrollView = new St.ScrollView({ style_class: 'nm-dialog-scroll-view' });
+        this._scrollView.set_x_expand(true);
+        this._scrollView.set_y_expand(true);
         this._scrollView.set_policy(Gtk.PolicyType.NEVER,
                                     Gtk.PolicyType.AUTOMATIC);
         this._scrollView.add_actor(this._itemBox);
-        this.contentLayout.add(this._scrollView);
+        this._stack.add_child(this._scrollView);
+
+        this._noNetworksLabel = new St.Label({ style_class: 'no-networks-label',
+                                               x_align: Clutter.ActorAlign.CENTER,
+                                               y_align: Clutter.ActorAlign.CENTER,
+                                               text: _("No Networks") });
+        this._stack.add_child(this._noNetworksLabel);
+
+        this.contentLayout.add(this._stack, { expand: true });
 
         this._disconnectButton = this.addButton({ action: Lang.bind(this, this.close),
                                                   label: _("Cancel"),
@@ -824,6 +868,8 @@ const NMWirelessDialog = new Lang.Class({
             this._createNetworkItem(network);
             this._itemBox.insert_child_at_index(network.item.actor, newPos);
         }
+
+        this._updateVisibility();
     },
 
     _accessPointRemoved: function(device, accessPoint) {
@@ -844,6 +890,8 @@ const NMWirelessDialog = new Lang.Class({
             network.item.updateBestAP(network.accessPoints[0]);
             this._resortItems();
         }
+
+        this._updateVisibility();
     },
 
     _resortItems: function() {
@@ -871,6 +919,7 @@ const NMWirelessDialog = new Lang.Class({
 
     _createNetworkItem: function(network) {
         network.item = new NMWirelessDialogItem(network);
+        network.item.setActive(network == this._selectedNetwork);
         network.item.connect('selected', Lang.bind(this, function() {
             Util.ensureActorVisibleInScrollView(this._scrollView, network.item.actor);
             this._selectNetwork(network);
@@ -1152,13 +1201,13 @@ Signals.addSignalMethods(NMVPNSection.prototype);
 
 const NMApplet = new Lang.Class({
     Name: 'NMApplet',
-    Extends: PanelMenu.SystemStatusButton,
+    Extends: PanelMenu.SystemIndicator,
 
     _init: function() {
-        this.parent('network-offline-symbolic', _('Network'));
+        this.parent();
 
-        this._vpnIcon = this.addIcon(null);
-        this._vpnIcon.hide();
+        this._primaryIndicator = this._addIndicator();
+        this._vpnIndicator = this._addIndicator();
 
         // Device types
         this._dtypes = { };
@@ -1204,25 +1253,22 @@ const NMApplet = new Lang.Class({
         this._nmDevices = [];
         this._devices = { };
 
-        this._section = new PopupMenu.PopupMenuSection();
-        this.menu.addMenuItem(this._section);
-
         this._devices.wireless = {
             section: new PopupMenu.PopupMenuSection(),
             devices: [ ],
         };
-        this._section.addMenuItem(this._devices.wireless.section);
+        this.menu.addMenuItem(this._devices.wireless.section);
 
         this._devices.wwan = {
             section: new PopupMenu.PopupMenuSection(),
             devices: [ ],
         };
-        this._section.addMenuItem(this._devices.wwan.section);
+        this.menu.addMenuItem(this._devices.wwan.section);
 
         this._vpnSection = new NMVPNSection(this._client);
         this._vpnSection.connect('activation-failed', Lang.bind(this, this._onActivationFailed));
         this._vpnSection.connect('icon-changed', Lang.bind(this, this._updateIcon));
-        this._section.addMenuItem(this._vpnSection.item);
+        this.menu.addMenuItem(this._vpnSection.item);
 
         this._readConnections();
         this._readDevices();
@@ -1242,7 +1288,7 @@ const NMApplet = new Lang.Class({
 
     _sessionUpdated: function() {
         let sensitive = !Main.sessionMode.isLocked && !Main.sessionMode.isGreeter;
-        this._section.setSensitive(sensitive);
+        this.menu.setSensitive(sensitive);
     },
 
     _ensureSource: function() {
@@ -1476,13 +1522,14 @@ const NMApplet = new Lang.Class({
             }
         }
 
-        this._mainConnection = activating || default_ip4 || default_ip6 || active_any || null;
+        this._mainConnection = default_ip4 || default_ip6 || active_any || activating || null;
 
         if (this._mainConnection) {
             let dev = this._mainConnection._primaryDevice;
             this._mainConnectionIconChangedId = dev.connect('icon-changed', Lang.bind(this, this._updateIcon));
-            this._updateIcon();
         }
+
+        this._updateIcon();
     },
 
     _notifyActivated: function(activeConnection) {
@@ -1572,35 +1619,35 @@ const NMApplet = new Lang.Class({
             this._vpnSection.checkConnection(connection);
         } else {
             let devices = this._devices[section].devices;
-            for (let i = 0; i < devices.length; i++) {
-                devices[i].checkConnection(connection);
-            }
+            devices.forEach(function(wrapper) {
+                if (wrapper instanceof NMConnectionSection)
+                    wrapper.checkConnection(connection);
+            });
         }
     },
 
     _syncNMState: function() {
-        this.mainIcon.visible = this._client.manager_running;
-        this.actor.visible = this.mainIcon.visible;
-
         this._syncActiveConnections();
-        this._section.actor.visible = this._client.networking_enabled;
+
+        this.indicators.visible = this._client.manager_running;
+        this.menu.actor.visible = this._client.networking_enabled;
     },
 
     _updateIcon: function() {
-        let hasApIcon = false;
-        let hasMobileIcon = false;
+        let mc = this._mainConnection;
 
-        if (!this._client.networking_enabled || !this._mainConnection) {
-            this.setIcon('network-offline-symbolic');
+        if (!this._client.networking_enabled || !mc) {
+            this._primaryIndicator.icon_name = 'network-offline-symbolic';
         } else {
             let dev = this._mainConnection._primaryDevice;
             if (!dev) {
                 log('Active connection with no primary device?');
                 return;
             }
-            this.setIcon(dev.getIndicatorIcon());
+            this._primaryIndicator.icon_name = dev.getIndicatorIcon(mc);
         }
 
-        this._vpnIcon.icon_name = this._vpnSection.getIndicatorIcon();
+        this._vpnIndicator.icon_name = this._vpnSection.getIndicatorIcon();
+        this._vpnIndicator.visible = (this._vpnIndicator.icon_name != '');
     }
 });
