@@ -25,10 +25,12 @@ const PowerManagerProxy = Gio.DBusProxy.makeProxyWrapper(PowerManagerInterface);
 
 const Indicator = new Lang.Class({
     Name: 'PowerIndicator',
-    Extends: PanelMenu.SystemStatusButton,
+    Extends: PanelMenu.SystemIndicator,
 
     _init: function() {
-        this.parent('battery-missing-symbolic', _("Battery"));
+        this.parent();
+
+        this._indicator = this._addIndicator();
 
         this._proxy = new PowerManagerProxy(Gio.DBus.session, BUS_NAME, OBJECT_PATH,
                                             Lang.bind(this, function(proxy, error) {
@@ -72,12 +74,12 @@ const Indicator = new Lang.Class({
 
         if (state == UPower.DeviceState.DISCHARGING) {
             // Translators: this is <hours>:<minutes> Remaining (<percentage>)
-            return _("%d\u2236%d Remaining (%d%%)".format(hours, minutes, percentage));
+            return _("%d\u2236%02d Remaining (%d%%)".format(hours, minutes, percentage));
         }
 
         if (state == UPower.DeviceState.CHARGING) {
             // Translators: this is <hours>:<minutes> Until Full (<percentage>)
-            return _("%d\u2236%d Until Full (%d%%)".format(hours, minutes, percentage));
+            return _("%d\u2236%02d Until Full (%d%%)".format(hours, minutes, percentage));
         }
 
         // state is one of PENDING_CHARGING, PENDING_DISCHARGING
@@ -104,16 +106,14 @@ const Indicator = new Lang.Class({
 
     _syncIcon: function() {
         let icon = this._proxy.Icon;
-        let hasIcon = false;
-
         if (icon) {
             let gicon = Gio.icon_new_for_string(icon);
-            this.setGIcon(gicon);
+            this._indicator.gicon = gicon;
             this._item.icon.gicon = gicon;
-            hasIcon = true;
+        } else {
+            // If there's no battery, then we use the power icon.
+            this._indicator.icon_name = 'system-shutdown-symbolic';
         }
-        this.mainIcon.visible = hasIcon;
-        this.actor.visible = hasIcon;
     },
 
     _sync: function() {
