@@ -42,14 +42,15 @@ const SystemdLoginSession = Gio.DBusProxy.makeProxyWrapper(SystemdLoginSessionIf
 
 const Indicator = new Lang.Class({
     Name: 'SystemIndicator',
-    Extends: PanelMenu.SystemStatusButton,
+    Extends: PanelMenu.SystemIndicator,
 
     _init: function() {
-        this.parent('system-shutdown-symbolic', _("System"));
+        this.parent();
 
         this._screenSaverSettings = new Gio.Settings({ schema: SCREENSAVER_SCHEMA });
         this._lockdownSettings = new Gio.Settings({ schema: LOCKDOWN_SCHEMA });
         this._privacySettings = new Gio.Settings({ schema: PRIVACY_SCHEMA });
+        this._orientationSettings = new Gio.Settings({ schema: 'org.gnome.settings-daemon.peripherals.touchscreen' });
 
         this._session = new GnomeSession.SessionManager();
         this._haveShutdown = true;
@@ -93,14 +94,40 @@ const Indicator = new Lang.Class({
         this._lockdownSettings.connect('changed::' + DISABLE_LOG_OUT_KEY,
                                        Lang.bind(this, this._updateHaveShutdown));
 
+        this._orientationSettings.connect('changed::orientation-lock',
+                                          Lang.bind(this, this._updateOrientationLock));
+        this._orientationExists = false;
+        Gio.DBus.session.watch_name('org.gnome.SettingsDaemon.Orientation',
+                                    Gio.BusNameWatcherFlags.NONE,
+                                    Lang.bind(this, function() {
+                                        this._orentationExists = true;
+                                        this._updateOrientationLock();
+                                    }),
+                                    Lang.bind(this, function() {
+                                        this._orentationExists = false;
+                                        this._updateOrientationLock();
+                                    }));
+        this._updateOrientationLock();
+
         Main.sessionMode.connect('updated', Lang.bind(this, this._sessionUpdated));
         this._sessionUpdated();
     },
 
+    _updateActionsVisibility: function() {
+        let visible = (this._settingsAction.visible ||
+                       this._orientationLockAction.visible ||
+                       this._lockScreenAction.visible ||
+                       this._powerOffAction.visible);
+
+        this._actionsItem.actor.visible = visible;
+    },
+
     _sessionUpdated: function() {
         this._updateLockScreen();
         this._updatePowerOff();
+        this._updateMultiUser();
         this._settingsAction.visible = Main.sessionMode.allowSettings;
+        this._updateActionsVisibility();
     },
 
     _updateMultiUser: function() {
@@ -135,6 +162,17 @@ const Indicator = new Lang.Class({
 
     _updateSwitchUserSubMenu: function() {
         this._switchUserSubMenu.label.text = this._user.get_real_name();
+        let clutterText = this._switchUserSubMenu.label.clutter_text;
+
+        // XXX -- for some reason, the ClutterText's width changes
+        // rapidly unless we force a relayout of the actor. Probably
+        // a size cache issue or something. Moving this to be a layout
+        // manager would be a much better idea.
+        clutterText.get_allocation_box();
+
+        let layout = clutterText.get_layout();
+        if (layout.is_ellipsized())
+            this._switchUserSubMenu.label.text = this._user.get_user_name();
 
         let iconFile = this._user.get_icon_file();
         if (iconFile && !GLib.file_test(iconFile, GLib.FileTest.EXISTS))
@@ -145,14 +183,25 @@ const Indicator = new Lang.Class({
             let gicon = new Gio.FileIcon({ file: file });
             this._switchUserSubMenu.icon.gicon = gicon;
         } else {
-            this._switchUserSubMenu.icon_name = 'avatar-default-symbolic';
+            this._switchUserSubMenu.icon.icon_name = 'avatar-default-symbolic';
         }
     },
 
+    _updateOrientationLock: function() {
+        this._orientationLockAction.visible = this._orientationExists;
+
+        let locked = this._orientationSettings.get_boolean('orientation-lock');
+        let icon = this._orientationLockAction.child;
+        icon.icon_name = locked ? 'rotation-locked-symbolic' : 'rotation-allowed-symbolic';
+
+        this._updateActionsVisibility();
+    },
+
     _updateLockScreen: function() {
         let showLock = !Main.sessionMode.isLocked && !Main.sessionMode.isGreeter;
         let allowLockScreen = !this._lockdownSettings.get_boolean(DISABLE_LOCK_SCREEN_KEY);
         this._lockScreenAction.visible = showLock && allowLockScreen && LoginManager.canLock();
+        this._updateActionsVisibility();
     },
 
     _updateHaveShutdown: function() {
@@ -167,6 +216,7 @@ const Indicator = new Lang.Class({
 
     _updatePowerOff: function() {
         this._powerOffAction.visible = this._haveShutdown && !Main.sessionMode.isLocked;
+        this._updateActionsVisibility();
     },
 
     _createActionButton: function(iconName, accessibleName) {
@@ -185,6 +235,15 @@ const Indicator = new Lang.Class({
         this._switchUserSubMenu = new PopupMenu.PopupSubMenuMenuItem('', true);
         this._switchUserSubMenu.icon.style_class = 'system-switch-user-submenu-icon';
 
+        // Since the label of the switch user submenu depends on the width of
+        // the popup menu, and we can't easily connect on allocation-changed
+        // or notify::width without creating layout cycles, simply update the
+        // label whenever the menu is opened.
+        this.menu.connect('open-state-changed', Lang.bind(this, function(menu, isOpen) {
+            if (isOpen)
+                this._updateSwitchUserSubMenu();
+        }));
+
         item = new PopupMenu.PopupMenuItem(_("Switch User"));
         item.connect('activate', Lang.bind(this, this._onLoginScreenActivate));
         this._switchUserSubMenu.menu.addMenuItem(item);
@@ -202,24 +261,26 @@ const Indicator = new Lang.Class({
 
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
-        let hbox = new St.BoxLayout({ style_class: 'system-menu-actions-box' });
+        item = new PopupMenu.PopupBaseMenuItem({ reactive: false,
+                                                 can_focus: false });
 
         this._settingsAction = this._createActionButton('preferences-system-symbolic', _("Settings"));
         this._settingsAction.connect('clicked', Lang.bind(this, this._onSettingsClicked));
-        hbox.add(this._settingsAction, { expand: true, x_fill: false });
+        item.actor.add(this._settingsAction, { expand: true, x_fill: false });
+
+        this._orientationLockAction = this._createActionButton('', _("Orientation Lock"));
+        this._orientationLockAction.connect('clicked', Lang.bind(this, this._onOrientationLockClicked));
+        item.actor.add(this._orientationLockAction, { expand: true, x_fill: false });
 
         this._lockScreenAction = this._createActionButton('changes-prevent-symbolic', _("Lock"));
         this._lockScreenAction.connect('clicked', Lang.bind(this, this._onLockScreenClicked));
-        hbox.add(this._lockScreenAction, { expand: true, x_fill: false });
+        item.actor.add(this._lockScreenAction, { expand: true, x_fill: false });
 
         this._powerOffAction = this._createActionButton('system-shutdown-symbolic', _("Power Off"));
         this._powerOffAction.connect('clicked', Lang.bind(this, this._onPowerOffClicked));
-        hbox.add(this._powerOffAction, { expand: true, x_fill: false });
-
-        item = new PopupMenu.PopupBaseMenuItem({ reactive: false,
-                                                 can_focus: false });
-        item.addActor(hbox, { expand: true });
+        item.actor.add(this._powerOffAction, { expand: true, x_fill: false });
 
+        this._actionsItem = item;
         this.menu.addMenuItem(item);
     },
 
@@ -230,6 +291,13 @@ const Indicator = new Lang.Class({
         app.activate();
     },
 
+    _onOrientationLockClicked: function() {
+        this.menu.itemActivated();
+        let locked = this._orientationSettings.get_boolean('orientation-lock');
+        this._orientationSettings.set_boolean('orientation-lock', !locked);
+        this._updateOrientationLock();
+    },
+
     _onLockScreenClicked: function() {
         this.menu.itemActivated(BoxPointer.PopupAnimation.NONE);
         Main.overview.hide();
