@@ -13,9 +13,6 @@ GType shell_app_state_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_APP_STATE (shell_app_state_get_type())
 
 /* enumerations from "./shell-global.h" */
-GType shell_stage_input_mode_get_type (void) G_GNUC_CONST;
-#define SHELL_TYPE_STAGE_INPUT_MODE (shell_stage_input_mode_get_type())
-
 GType shell_cursor_get_type (void) G_GNUC_CONST;
 #define SHELL_TYPE_CURSOR (shell_cursor_get_type())
 
